<?php

/**
 * The class responsible for defining all actions that occur in the admin area.
 */
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-ctr-helpers.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-ctr-db.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-ctr-media.php';
require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-ctr-templator.php';

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.chiptuningreseller.com
 * @since      3.0.0
 *
 * @package    Ctr
 * @subpackage Ctr/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Ctr
 * @subpackage Ctr/admin
 * @author     M Bos <support@chiptuningreseller.com>
 */
class Ctr_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	private $helper;

	private $database;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->helpers = new Ctr_Helpers();
		$this->media = new CTR_Media();
		$this->templator = new CTR_Templator();
		$this->database = new Ctr_DB();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Ctr_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Ctr_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style('wp-color-picker');
		wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/ctr-admin.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * Enqueue all needed scripts
		 */

		wp_enqueue_script('jquery', plugin_dir_url(__FILE__) . 'js/jquery.min.js', array('jquery'), $this->version, false);
		wp_enqueue_script('ctr-admin', plugin_dir_url(__FILE__) . 'js/ctr-admin.js', array('jquery'), $this->version, false);
		wp_enqueue_script('admin', plugin_dir_url(__FILE__) . 'js/admin.js', array('jquery'), $this->version, false);
		wp_enqueue_script('colorpick', plugin_dir_url(__FILE__) . 'js/alpha-color-picker.js', array('wp-color-picker'), false, false);
		
		
		$cm_settings['codeEditor'] = wp_enqueue_code_editor(array('type' => 'application/x-httpd-php'));
		wp_localize_script('jquery', 'cm_settings', $cm_settings);

	}

	/**
	 * add_admin_menu
	 * 
	 * @access public
	 *
	 * @return mixed Value.
	 */
	public function create_admin_menu()
	{
		$notification = 'ChiptuningReseller';

		$hook = add_menu_page(
			'ChiptuningReseller',
			$notification,
			'manage_options',
			'__chiptuningreseller',
			array($this, 'settings_page')
		);
	}


	/** 
	 * Show the settings page on request
	 *  */
	public function settings_page()
	{

		// We want to save our settings - the must be done before getting them (in some cases the db is not properly created)
		if (($_POST) && ($_POST['submit'] == 'Save')) {
			$_POST['submit'] = null; // reset post so we don't get into a loop and show the notice :)
			$this->save_settings($_POST);
		}
		
		$notices = [];
		$catalogData = $this->database->getSettings();
		$catalogData['api_key'] = $this->database->getApiKey();
		$catalogData['connection_valid'] = $this->helpers->check_api_connection();
		$catalogData['languages'] = $this->helpers->get_languages();

		if (!$catalogData['connection_valid']) {
			$this->helpers->show_notice("error", __("API connection failed!", "ctr") . '<br/>' . __("If you have accidentally changed the API key, please restore it ASAP, because this will significantly decrease your SEO positions!", "ctr"));
		}

		if(is_plugin_active('wp-rocket/wp-rocket.php')) {
			$this->helpers->show_notice("error", __("WP Rocket Detected", "ctr") . '<br/>' . sprintf( __( 'Please make sure to set exclusion for ChiptuningReseller, <a href="%s" target="_blank">read more here</a> - if you already did, ignore this message', 'chiptuningreseller' ), 
            'https://docs.chiptuningreseller.com/wp-plugin#wprocket'
        ));
		}

		$catalogData['template_files'] = $this->list_template_files();

		$tab = (!empty($_GET['tab'])) ? esc_attr($_GET['tab']) : 'first';
		echo $this->create_page_tabs($tab);

		if ($tab == 'first') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-display-settings.php';
		}
		if ($tab == 'second') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-display-customisation.php';
		}
		if ($tab == 'third') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-display-prices.php';
		}
		if ($tab == 'fourth') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-template-editor.php';
		}
		if ($tab == 'fifth') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-seo.php';
		}
		if ($tab == 'sixth') {
			require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/ctr-admin-help.php';
		}
	}

	/** Save the settings to the database */
	public function save_settings($postData)
	{
		if ($postData['form_type'] == 'template_file') {
			// Update a template file
			if (isset($postData['file_update'])) {
				$file_location = wp_upload_dir()['basedir'] . '/ctr/custom_templates/' . $postData['file_name'];
				if (!file_exists($file_location)) {
					$file_location = $postData['file_name'];
				}
				// Find the template file
				if (!file_exists($file_location)) {
					$helper = new Ctr_Helpers();
					$helper->show_notice("error", __("File not found!", "ctr") . '<br/>The file ' . $file_location . ' you wanted to update does not exist, please try again.', "ctr");
					$response = false;
				}
				// Save the data to the template file
				if (file_put_contents($file_location, stripslashes($postData['file_update']))) {
					$response = true;
				}
			}
			if($response == true) {
				/** Clear the cached templates so the changes will become effectively */
				$this->templator->clear_cache();
			}
		} else {
			if ($postData['form_type'] == 'seo') {
				$this->check_seo_settings_column();
			}

			$response = $this->database->saveSettings($postData);
		}
		if ($response !== false) {
			// We updated our main url, we want to flush the rewrites on a next 'init' event
			if(isset($postData['start_url'])) {
				add_option('ctr_flush_rewrites', true);
			}
			$this->helpers->show_notice("success", __("Settings saved", "ctr"));
			$this->settings_page(); // reload the settings page
		} else {
			$this->helpers->show_notice("error", __("Could not save settings", "ctr"));
		}
	}

	/** 
	 * Create the Page Tabs
	 *  */
	public function create_page_tabs($current = 'first')
	{
		$tabs = array(
			'first'   => __('Main settings', 'chiptuningreseller'),
			'second'  => __('Layout & Customisation', 'chiptuningreseller'),
			'third'  => __('Price settings', 'chiptuningreseller'),
			'fourth'  => __('Template editor', 'chiptuningreseller'),
			'fifth'  => __('SEO Settings', 'chiptuningreseller'),
			'sixth'  => __('Help & Support', 'chiptuningreseller'),
		);
		$html = '<h2 class="nav-tab-wrapper">';
		foreach ($tabs as $tab => $name) {
			$class = ($tab == $current) ? 'nav-tab-active' : '';
			$html .= '<a class="nav-tab ' . $class . '" href="?page=__chiptuningreseller&tab=' . $tab . '">' . $name . '</a>';
		}
		$html .= '</h2>';
		return $html;
	}

	/** Flush all cache */
	public function flush_cache()
	{
		global $wpdb;
		if ($wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name LIKE ('%_ctr_%');")) {
			echo 'All CTR cache items have been deleted!';
			wp_die();
		}
	}

	/** Flush all media */
	public function flush_media()
	{
		global $wpdb;
		if ($this->media->clear_media(true)) {
			if(is_numeric($wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name LIKE ('%_ctr_media_%');"))) {
				echo 'All media items and the media cache have been deleted. Please visit your site again to re-load all images or let the visitors do the work for you. The website can be a bit slower due to first-time loading. Sometimes it needs some time to correctly load all images and therefore you may see the placeholder in some occassions.';
				wp_die();
			}
		} else {
			echo 'Something went wrong. Please check manually if the items have been deleted or try again.';
		}
	}

	/** Flush all cached blade templates */
	public function flush_templates()
	{
		if ($this->templator->clear_cache()) {
			echo 'All the cached pages have been removed!';
			wp_die();
		} else {
			echo 'Something went wrong!';
		}
	}


	/** Copy the default templates to the default theme folder */
	public function copy_templates()
	{
		$themeFolder = wp_upload_dir()['basedir'] . '/ctr/custom_templates';
		$copied = $this->templator->copy_templates($themeFolder);
		if ($copied != false) {
			if ($copied === 0) {
				echo "\r\nThe template files have been copied to the folder " . $themeFolder . " - edit your templates in the TEMPLATE EDITOR";
			} else {
				echo "\r\n$copied templates already existed. Please add / edit them manually!";
			}
			wp_die();
		} else {
			echo 'Something went wrong!';
		}
	}


	/**
	 * RESET a template file
	 */
	public function reset_template_file()
	{
		$file_name = esc_attr($_POST['file_name']);
		// Reset a template file
		$original_file = dirname(__DIR__) . '/public/templates/' . $file_name;
		$destination = wp_upload_dir()['basedir'] . '/ctr/custom_templates/' . $file_name;
		if (file_exists($original_file)) {
			if (copy($original_file, $destination)) {
				echo "\r\nThe original template file ($file_name) has been restored!";
				wp_die();
			}
		} else {
			echo 'Something went wrong!';
		}
	}

	/**
	 * List the template files for use in our editor
	 */
	public function list_template_files()
	{
		
		$found_files = [];
		$custom_template_folder = wp_upload_dir()['basedir'] . '/ctr/custom_templates/';
		if(!is_dir($custom_template_folder)) {
			return false;
		}
		$rii = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($custom_template_folder));

		$files = array();
		$i = 0;
		$dir = null;

		foreach ($rii as $file) {
			
			if ($dir !== basename($file->getPath())) {
				$dir = basename($file->getPath());
			} else {
				$files[$i]['dir'] = $dir;
			}

			/** Only add the file to the list when it is not a directory */
			if(!$file->isDir()) {
				$files[$i]['dir'] = $dir;
				$files[$i]['file'] = $file->getPathname();
				$i++;
			}
		}
		return $files;
	}

	/**
	 * @since 3.3.9.6
	 * Check if our SEO Settings Column has already been added. If not, add it.
	 */
	public function check_seo_settings_column() {
		global $wpdb;
		$row = $wpdb->query("SHOW COLUMNS FROM `{$wpdb->prefix}chiptuningreseller` LIKE 'seo_settings'");
		if(null == $row){
			$wpdb->query("ALTER TABLE {$wpdb->prefix}chiptuningreseller ADD `seo_settings` MEDIUMTEXT NULL AFTER `layout_settings`");
		}
	}
}